let game;
let onGameLoadComplete;
let backgroundAudio;
let audioPlayer;
let gameOver = false;
let audioPath = {
  bgAudio: "./assets/audios/Music_Ketchup-Stash.mp3",
  jumpAudio: "./assets/audios/Jump.mp3",
};
var res = 1;

// global game options
let gameOptions = {
  // platform speed range, in pixels per second
  platformSpeedRange: [400, 400],

  // mountain speed, in pixels per second
  backgroundTrailSpeed: 80,

  // spawn range, how far should be the rightmost platform from the right edge
  // before next platform spawns, in pixels
  spawnRange: [400, 500],

  // platform width range, in pixels
  platformSizeRange: [400, 700],

  // a height range between rightmost platform and next platform to be spawned
  platformHeightRange: [-5, 5],

  // a scale to be multiplied by platformHeightRange
  platformHeighScale: 20,

  // platform max and min height, as screen height ratio
  platformVerticalLimit: [0.6, 0.8],

  // player gravity
  playerGravity: 1100,

  // player jump force
  jumpForce: 600,

  // player starting X position
  playerStartPosition: 400,

  // consecutive jumps allowed
  jumps: 2,

  // % of probability a coin appears on the platform
  coinPercent: 75,
  bookPercent: 75,

  gameImages: {
    gameTrigger: {
      url: "assets/images/level-1/game_trigger.png",
      displaySize: [32, 65],
      imageSize: [63, 130],
    },
    gameTrigger1: {
      url: "assets/images/level-1/info_image.png",
      displaySize: [50, 75],
      imageSize: [63, 130],
    },
    background: "assets/images/level-1/background.png",
    platform: "assets/images/level-1/platform.png",
  },
};

$(document).ready(function () {
  backgroundAudio = new loadGameBgAudio(audioPath.bgAudio, 0.2);
  audioPlayer = new loadAudioPlayer();
  initGame();
  showPreloader(false);
  $("#startGameBtn")
    .off()
    .on("click", function () {
      showPreloader(true);
      $(this).hide();
      setTimeout(function () {
        startGame(function () {
          showPreloader(false);
        });
      }, 1500);
    });
});

function resumeGame() {
  game.scene.keys.PlayGame.player.anims.play("run");
  game.scene.keys.PlayGame.pauseGame = false;
  backgroundAudio.playBg();
}

function pauseGame(isPause) {
  game.scene.keys.PlayGame.pauseGame = isPause;
  game.scene.keys.PlayGame.isOnPlatform = false;
}

function initGame(_gameImages, options) {
  // gameOptions.gameImages = _gameImages;
  if (game) {
    game.destroy();
    game = null;
    onGameLoadComplete = null;
    $("#gameCanvas").find("canvas").remove();
  } else {
    window.addEventListener("resize", resize, false);
  }

  // object containing configuration options
  let gameConfig = {
    type: Phaser.AUTO,
    scale: {
      mode: Phaser.Scale.FIT,
      autoCenter: Phaser.Scale.CENTER_BOTH,
      parent: "gameCanvas",
      width: 1246,
      height: 700,
    },
    scene: [preloadGame, playGame],
    backgroundColor: 0xd6f5f8,

    // physics settings
    physics: {
      default: "arcade",
    },
  };
  onGameLoadComplete = options;
  game = new Phaser.Game(gameConfig);
  window.focus();
  resize();
}

function startGame(option) {
  if (game) {
    game.scene.keys.PlayGame.pauseGame = false;
    game.scene.keys.PlayGame.isJumpping = false;
    game.scene.keys.PlayGame.isOnPlatform = true;
    game.scene.start("PlayGame");
    backgroundAudio.playBg();
    showPreloader(false);
  }
  if (option && typeof option === "function") {
    option();
  }
}

// preloadGame scene
class preloadGame extends Phaser.Scene {
  constructor() {
    super("PreloadGame");
  }
  preload() {
    this.load.image("backgroundTrail", gameOptions.gameImages.background);
    this.load.image("platform", gameOptions.gameImages.platform);
    let graphics = this.add.graphics();
    graphics.fillGradientStyle(0xb7eaff, 0xb7eaff, 0x94dfff, 0xc1e6e9, 1);
    graphics.fillRect(0, 0, game.config.width, game.config.height);
    // player is a sprite sheet made by 24x48 pixels
    this.load.spritesheet("player", "./assets/images/KoolAidMan_Sprite.png", {
      frameWidth: 214,
      frameHeight: 231,
    });

    // the coin is a sprite sheet made by 28x56 pixels
    this.load.spritesheet("coin", gameOptions.gameImages.gameTrigger.url, {
      frameWidth: gameOptions.gameImages.gameTrigger.imageSize[0],
      frameHeight: gameOptions.gameImages.gameTrigger.imageSize[1],
    });
    this.load.spritesheet("book", gameOptions.gameImages.gameTrigger1.url, {
      frameWidth: gameOptions.gameImages.gameTrigger1.imageSize[0],
      frameHeight: gameOptions.gameImages.gameTrigger1.imageSize[1],
    });
    this.load.on("complete", function () {
      if (onGameLoadComplete && typeof onGameLoadComplete === "function") {
        onGameLoadComplete();
      }
    });
  }
  create() {
    // setting player animation
    this.anims.create({
      key: "stop",
      frames: this.anims.generateFrameNumbers("player", {
        start: 0,
        end: 0,
      }),
      frameRate: 0,
      repeat: -1,
    });
    this.anims.create({
      key: "run",
      frames: this.anims.generateFrameNumbers("player", {
        start: 0,
        end: 3,
      }),
      frameRate: 13,
      repeat: -1,
    });

    this.anims.create({
      key: "jump",
      frames: this.anims.generateFrameNumbers("player", {
        start: 4,
        end: 7,
      }),
      frameRate: 4,
      repeat: -1,
    });

    this.anims.create({
      key: "jumpDbl",
      frames: this.anims.generateFrameNumbers("player", {
        start: 5,
        end: 6,
      }),
      frameRate: 2,
      repeat: -1,
    });

    this.anims.create({
      key: "fall",
      frames: this.anims.generateFrameNumbers("player", {
        start: 8,
        end: 11,
      }),
      frameRate: 8,
      repeat: -1,
    });

    // setting coin animation
    this.anims.create({
      key: "rotate",
      frames: this.anims.generateFrameNumbers("coin", {
        start: 0,
        end: 0,
      }),
      frameRate: 1,
      repeat: -1,
    });
    //setting book animation
    this.anims.create({
      key: "rotate",
      frames: this.anims.generateFrameNumbers("book", {
        start: 30,
        end: 50,
      }),
      frameRate: 1,
      repeat: -1,
    });
  }
}

// playGame scene
class playGame extends Phaser.Scene {
  constructor() {
    super("PlayGame");
  }

  create() {
    /*  this.pauseGame = false; */
    this.gameOver = false;

    // group with all active platforms.
    this.platformGroup = this.add.group({
      // once a platform is removed, it's added to the pool
      removeCallback: function (platform) {
        platform.scene.platformPool.add(platform);
      },
    });

    // platform pool
    this.platformPool = this.add.group({
      // once a platform is removed from the pool, it's added to the active platforms group
      removeCallback: function (platform) {
        platform.scene.platformGroup.add(platform);
      },
    });

    // group with all active coins.
    this.coinGroup = this.add.group({
      // once a coin is removed, it's added to the pool
      removeCallback: function (coin) {
        coin.scene.coinPool.add(coin);
      },
    });

    // group with all active coins.
    this.bookGroup = this.add.group({
      // once a coin is removed, it's added to the pool
      removeCallback: function (book) {
        book.scene.bookGroup.add(book);
      },
    });
    // coin pool
    this.coinPool = this.add.group({
      // once a coin is removed from the pool, it's added to the active coins group
      removeCallback: function (coin) {
        coin.scene.coinGroup.add(coin);
      },
    });

    this.bookPool = this.add.group({
      // once a coin is removed from the pool, it's added to the active coins group
      removeCallback: function (book) {
        book.scene.bookGroup.add(book);
      },
    });
    this.backgroundTrail = this.add.tileSprite(
      game.config.width / 2,
      game.config.height / 2,
      4076,
      700,
      "backgroundTrail"
    );
    this.backgroundTrail.setDepth(1);
    // this.backgroundTrail.skewY(1);

    // keeping track of added platforms
    this.addedPlatforms = 0;

    // number of consecutive jumps made by the player so far
    this.playerJumps = 0;

    // adding a platform to the game, the arguments are platform width, x position and y position
    this.addPlatform(
      game.config.width,
      game.config.width / 2,
      game.config.height * gameOptions.platformVerticalLimit[1]
    );

    // adding the player;
    this.player = this.physics.add.sprite(
      gameOptions.playerStartPosition,
      game.config.height * gameOptions.platformVerticalLimit[1] - 110,
      "player"
    );
    this.player.setGravityY(gameOptions.playerGravity);
    this.player.setDepth(3);
    this.player.displayWidth = 100;
    this.player.displayHeight = 112;

    // the player is not dying
    this.dying = false;

    // the player ison platform
    this.isOnPlatform = false;

    // the player is jumpping
    this.isJumpping = true;

    // setting collisions between the player and the platform group
    this.platformCollider = this.physics.add.collider(
      this.player,
      this.platformGroup,
      function () {
        // play "run" animation if the player is on a platform
        this.isJumpping = false;
        this.isFallingTriggered = true;
        if (!this.pauseGame && !this.isOnPlatform) {
          this.player.anims.play("run");
          this.isOnPlatform = true;
        }
      },
      null,
      this
    );

    // setting collisions between the player and the coin group
    this.physics.add.overlap(
      this.player,
      this.coinGroup,
      function (player, coin) {
        if (!this.pauseGame) {
          this.player.anims.play("stop");
          this.pauseGame = true;
          // App.assessments.coinAnimation(this.tweens, coin, this);
          backgroundAudio.pauseBg();
          $(".gamezone").css("width", "100%");
          $(".game-based-temp").css("width", "50%");
          $(".game-based-temp").css("position", "absolute");
          $(".footer-bottom-div").css("width", "50%");
          console.log("Add Question Here");
          console.log(
            "After Question attempt, to start game use this function to run game again :: startGame();"
          );
          this.coinGroup.killAndHide(coin);
          this.coinGroup.remove(coin);

          // After Question attempt, to start game use this function to run game
          // startGame();
        }
      },
      null,
      this
    );

    this.physics.add.overlap(
      this.player,
      this.bookGroup,
      function (player, book) {
        if (!this.pauseGame) {
          this.player.anims.play("stop");
          this.pauseGame = true;
          // App.assessments.coinAnimation(this.tweens, coin, this);
          backgroundAudio.pauseBg();
          $(".gamezone").css("width", "100%");
          $(".game-based-temp").css("width", "50%");
          $(".game-based-temp").css("position", "absolute");
          $(".footer-bottom-div").css("width", "50%");
          console.log("Add information");
          console.log(
            "After Question attempt, to start game use this function to run game again :: startGame();"
          );
          this.bookGroup.killAndHide(book);
          this.bookGroup.remove(book);
          // setTimeout(() => {
          //   resumeGame();
          // }, 4000);
          // this.scene.play("run", {});

          // After Question attempt, to start game use this function to run game
          // startGame();
        }
      },
      null,
      this
    );
    // checking for input
    this.input
      .off("pointerdown", this.jump, this)
      .on("pointerdown", this.jump, this);

    this.input.keyboard
      .off("keydown_SPACE", this.jump, this)
      .on("keydown_SPACE", this.jump, this);

    this.input.keyboard
      .off("keydown_UP", this.jump, this)
      .on("keydown_UP", this.jump, this);

    /*  this.input.keyboard.on("keydown_LEFT", function () {
             this.pauseGame = true;
         }, this);
         this.input.keyboard.on("keydown_RIGHT", function () {
             this.pauseGame = false;
         }, this); */
  }

  // getting rightmost mountain x position
  getRightmostMountain() {
    let rightmostMountain = -1024;
    this.mountainGroup.getChildren().forEach(function (mountain) {
      rightmostMountain = Math.max(rightmostMountain, mountain.x);
    });
    return rightmostMountain;
  }

  // the core of the script: platform are added from the pool or created on the fly
  addPlatform(platformWidth, posX, posY) {
    this.addedPlatforms++;
    let platform;
    if (this.platformPool.getLength()) {
      platform = this.platformPool.getFirst();
      platform.x = posX;
      platform.y = posY;
      platform.active = true;
      platform.visible = true;
      this.platformPool.remove(platform);
      let newRatio = platformWidth / platform.displayWidth;
      platform.displayWidth = platformWidth;
      platform.tileScaleX = 1 / platform.scaleX;
    } else {
      platform = this.add.tileSprite(
        posX,
        posY,
        platformWidth,
        100,
        "platform"
      );
      this.physics.add.existing(platform);
      platform.body.setImmovable(true);
      platform.body.setVelocityX(
        Phaser.Math.Between(
          gameOptions.platformSpeedRange[0],
          gameOptions.platformSpeedRange[1]
        ) * -1
      );
      platform.setDepth(2);
      this.platformGroup.add(platform);
    }
    this.nextPlatformDistance = Phaser.Math.Between(
      gameOptions.spawnRange[0],
      gameOptions.spawnRange[1]
    );

    // if this is not the starting platform...
    if (this.addedPlatforms > 1) {
      // is there a coin over the platform?
      if (Phaser.Math.Between(1, 100) <= gameOptions.coinPercent) {
        if (this.coinPool.getLength()) {
          let coin = this.coinPool.getFirst();
          coin.x = posX;
          coin.y = posY - 96;
          coin.alpha = 1;
          coin.active = true;
          coin.visible = true;
          coin.displayWidth = gameOptions.gameImages.gameTrigger.displaySize[0];
          coin.displayHeight =
            gameOptions.gameImages.gameTrigger.displaySize[1];
          this.coinPool.remove(coin);
        } else {
          let coin = this.physics.add.sprite(posX, posY - 96, "coin");
          coin.setImmovable(true);
          coin.setVelocityX(platform.body.velocity.x);
          coin.displayWidth = gameOptions.gameImages.gameTrigger.displaySize[0];
          coin.displayHeight =
            gameOptions.gameImages.gameTrigger.displaySize[1];
          //coin.anims.play("rotate");
          coin.setDepth(2);
          this.coinGroup.add(coin);
        }
      }
      if (Phaser.Math.Between(1, 100) <= gameOptions.bookPercent) {
        if (this.bookPool.getLength()) {
          let book = this.bookPool.getFirst();
          console.log("book coordinates----", posX, posY);
          book.x = posX - 1000;
          book.y = posY - 140;
          book.alpha = 1;
          book.active = true;
          book.visible = true;
          book.displayWidth =
            gameOptions.gameImages.gameTrigger1.displaySize[0];
          book.displayHeight =
            gameOptions.gameImages.gameTrigger1.displaySize[1];
          this.bookPool.remove(book);
        } else {
          let book = this.physics.add.sprite(posX - 200, posY - 100, "book");
          book.setImmovable(true);
          book.setVelocityX(platform.body.velocity.x);
          book.displayWidth =
            gameOptions.gameImages.gameTrigger1.displaySize[0];
          book.displayHeight =
            gameOptions.gameImages.gameTrigger1.displaySize[1];
          //coin.anims.play("rotate");
          book.setDepth(2);
          this.bookGroup.add(book);
        }
      }
    }
  }

  // the player jumps when on the ground, or once in the air as long as there are jumps left and the first jump was on the ground
  // and obviously if the player is not dying
  jump() {
    if (
      !this.pauseGame &&
      !this.dying &&
      (this.player.body.touching.down ||
        (this.playerJumps > 0 && this.playerJumps < gameOptions.jumps))
    ) {
      if (this.player.body.touching.down) {
        this.isOnPlatform = false;
        this.playerJumps = 0;
        this.isJumpping = true;
        this.player.anims.play("jump");
        this.isFallingTriggered = true;
      } else {
        this.player.anims.play("jumpDbl");
      }
      this.player.setVelocityY(gameOptions.jumpForce * -1);
      this.playerJumps++;
      audioPlayer.loadAudio(audioPath.jumpAudio);
    }
  }

  update() {
    // game over
    if (this.gameOver) {
      this.platformGroup.getChildren().forEach(function (platform) {
        platform.body.setVelocityX(Phaser.Math.Between(0, 0));
      });
      this.coinGroup.getChildren().forEach(function (coin) {
        coin.body.setVelocityX(0);
      }, this);
      this.bookGroup.getChildren().forEach(function (book) {
        console.log("in update");
        book.body.setVelocityX(0);
      }, this);
      this.player.anims.stop();
      return;
    }
    if (!this.pauseGame && this.player.y > game.config.height) {
      this.gameOver = true;
      if (gameOver) {
        pauseGame(true);
        backgroundAudio.pauseBg();
      } else {
        /**
         * This is a patch work for game pre run while launch
         * this below block of code will only execute when course launch first time and game run in background
         * for eliminate player image loading bug.
         *
         */
        /* if (game) {
                    game.destroy();
                    game = null;
                    onGameLoadComplete = null;
                    $("#gameCanvas").find("canvas").remove();
                } */
      }
    }

    if (this.pauseGame) {
      this.player.x = gameOptions.playerStartPosition;
      this.platformGroup.getChildren().forEach(function (platform) {
        platform.body.setVelocityX(0);
      });
      this.player.anims.stop();
      this.coinGroup.getChildren().forEach(function (coin) {
        coin.body.setVelocityX(0);
      }, this);
      this.bookGroup.getChildren().forEach(function (book) {
        book.body.setVelocityX(0);
      }, this);
      return;
    } else {
      this.platformGroup.getChildren().forEach(function (platform) {
        platform.body.setVelocityX(
          Phaser.Math.Between(
            gameOptions.platformSpeedRange[0],
            gameOptions.platformSpeedRange[1]
          ) * -1
        );
      });
      this.coinGroup.getChildren().forEach(function (coin) {
        coin.body.setVelocityX(
          Phaser.Math.Between(
            gameOptions.platformSpeedRange[0],
            gameOptions.platformSpeedRange[1]
          ) * -1
        );
      }, this);
      this.bookGroup.getChildren().forEach(function (book) {
        book.body.setVelocityX(
          Phaser.Math.Between(
            gameOptions.platformSpeedRange[0],
            gameOptions.platformSpeedRange[1]
          ) * -1
        );
      }, this);
      // this.player.anims.play();
    }

    this.player.x = gameOptions.playerStartPosition;
    this.backgroundTrail.tilePositionX += 2;
    /* this.player.anims.isPlaying = false;
        this.player.anims.play("run"); */
    // recycling platforms
    let minDistance = game.config.width;
    let rightmostPlatformHeight = 0;
    let bottomMostPlatform = 0;
    this.platformGroup.getChildren().forEach(function (platform) {
      let platformDistance =
        game.config.width - platform.x - platform.displayWidth / 2;
      if (platformDistance < minDistance) {
        minDistance = platformDistance;
        rightmostPlatformHeight = platform.y;
      }
      if (platform.x < -platform.displayWidth / 2) {
        this.platformGroup.killAndHide(platform);
        this.platformGroup.remove(platform);
      }
      if (bottomMostPlatform < platform.y) {
        bottomMostPlatform = platform.y;
      }
    }, this);
    if (this.player.y > bottomMostPlatform && this.isFallingTriggered) {
      this.isFallingTriggered = false;
      this.player.anims.play("fall");
    }
    // recycling coins
    this.coinGroup.getChildren().forEach(function (coin) {
      if (coin.x < -coin.displayWidth / 2) {
        this.coinGroup.killAndHide(coin);
        this.coinGroup.remove(coin);
      }
    }, this);
    this.bookGroup.getChildren().forEach(function (book) {
      // console.log("in get children book ----", book);
      if (book.x < -book.displayWidth / 2) {
        console.log("book", book.x);
        this.bookGroup.killAndHide(book);
        this.bookGroup.remove(book);
      }
    }, this);

    // adding new platforms
    if (minDistance > this.nextPlatformDistance) {
      let nextPlatformWidth = Phaser.Math.Between(
        gameOptions.platformSizeRange[0],
        gameOptions.platformSizeRange[1]
      );
      let platformRandomHeight =
        gameOptions.platformHeighScale *
        Phaser.Math.Between(
          gameOptions.platformHeightRange[0],
          gameOptions.platformHeightRange[1]
        );
      let nextPlatformGap = rightmostPlatformHeight + platformRandomHeight;
      let minPlatformHeight =
        game.config.height * gameOptions.platformVerticalLimit[0];
      let maxPlatformHeight =
        game.config.height * gameOptions.platformVerticalLimit[1];
      let nextPlatformHeight = Phaser.Math.Clamp(
        nextPlatformGap,
        minPlatformHeight,
        maxPlatformHeight
      );
      this.addPlatform(
        nextPlatformWidth,
        game.config.width + nextPlatformWidth / 2,
        nextPlatformHeight
      );
    }
  }
}

function resize() {
  //   let canvas = document.querySelector("canvas");
  //   // let mainPage = document.querySelector("body");
  //   var gameWindow = document.getElementById("gameAppMain");
  //   let gameWindowWidth = gameWindow.innerWidth;
  //   let gameWindowHeight = gameWindow.innerHeight;
  //   let windowRatio = gameWindowWidth / gameWindowHeight;
  //   let gameRatio = game.config.width / game.config.height;
  //   // mainPage.style.width = window.innerWidth + "px";
  //   // mainPage.style.height = window.innerHeight + "px";
  //   if (windowRatio < gameRatio) {
  //     canvas.style.width = gameWindowWidth + "px";
  //     canvas.style.height = gameWindowWidth / gameRatio + "px";
  //   } else {
  //     canvas.style.width = gameWindowHeight * gameRatio + "px";
  //     canvas.style.height = gameWindowHeight + "px";
  //   }
  //   $(".assessment-container.app-container").width(canvas.style.width);
  //   $(".assessment-container.app-container").height(canvas.style.height);
}

let loadGameBgAudio = function (audioPath, volume) {
  let self = this;

  self.player = $("#gameBgSound")[0];
  self.isPlaying = false;
  let BgPlayer = self.player;
  if (audioPath) {
    BgPlayer.src = audioPath;
  }
  if (volume) {
    BgPlayer.volume = volume;
  }
  self.loadBgAudio = function (audioPath) {
    BgPlayer.pause();
    BgPlayer.src = audioPath;
  };
  self.playBg = function (restart) {
    if (restart) {
      BgPlayer.currentTime = 0;
    }
    try {
      BgPlayer.play();
    } catch (err) {}
    self.isPlaying = true;
  };
  self.pauseBg = function () {
    BgPlayer.pause();
    self.isPlaying = false;
  };

  self.muteUnmute = function (val) {
    self.player.muted = !self.player.muted;
  };
};

let loadAudioPlayer = function () {
  let self = this;

  self.player = $("#audioPlayer")[0];
  self.isPlaying = false;
  let Player = self.player;
  self.isCallback = null;

  self.loadAudio = function (audioPath, justLoadAudio) {
    Player.pause();
    Player.src = audioPath;
    if (!justLoadAudio) {
      self.playAudio();
    } else {
      if (typeof justLoadAudio === "function") {
        self.isCallback = justLoadAudio;
      }
    }
  };

  self.playAudio = function () {
    try {
      Player.play();
    } catch (err) {}
    self.isPlaying = true;
  };
  self.pauseAudio = function () {
    Player.pause();
    self.isPlaying = false;
  };

  Player.onended = function () {
    // Player.src = "assets/audios/silent_audio.mp3";
    self.isPlaying = false;
  };

  Player.onloadeddata = function () {
    // console.log("Assets loaded ", Player.src);
    if (typeof self.isCallback === "function") {
      self.isCallback();
    }
  };

  self.muteUnmute = function (val) {
    self.player.muted = !self.player.muted;
  };
};

function showPreloader(show) {
  if (show) {
    $("#preLoader").show();
  } else {
    $("#preLoader").hide();
  }
}
